--[[
	##AddOn	  : CooldownIcons R(evamped)
	##Subfile : LibCooldownIcons-1.0.lua
	##Author  : Hati-EK
--]]


local MAJOR, MINOR = "LibCooldownIcons", "1"
local lib = LibStub:NewLibrary(MAJOR, MINOR)
local CIR = CooldownIconsR

if not lib then return end

local Icon_Nil = "Interface\\Icons\\INV_Misc_QuestionMark"
-- i = LCI:tonumber <tonumber>, i<-R
-- i = LCI:tonumber (extended tonumber), i<-R
function lib:tonumber(s)
	if type(s)=='boolean' then
		if s then
			return 1
		else
			return 0
		end
	else
		return tonumber(s)
	end
end

-- i = LCI:Getn(table), i<-N
function lib:Getn(t)
	local i=0
	for k,v in pairs(t) do
		i=i+1
	end
	return i
end

-- b,i = LCI:inTable(v)| b<-{false,true}, i<-{v[indexes]}
function lib:inTable(value,tbl)
	local b=false
	local num=0
	if type(tbl)~='table' and value==nil then
		error('Failure in arguments. Arguments:( value='..tostring(value)..', table='..tostring(tbl)..')')
		return false,nil
	end
	if type(tbl)=='table' then
		for i,v in pairs(tbl) do
			if v==value then
				b=true
				num=i
			end
		end
	end
	return b,num
end

function lib:indexInTable(value,tbl)
  local b=false
  local index=0
  if type(tbl)~='table' or value==nil then
    error('Failure in arguments. Arguments:( value='..tostring(value)..', table='..tostring(tbl)..')')
    return false,nil
  end
  for i,v in pairs(tbl) do
    if i==value then
      b=true
      index=i
    end
  end
  return b,index
end


--##Texture Part
-- i = LCI:GetTexture(name), i<-TexturePath
function lib:GetTexture(name)
	return Icon_Nil
end


function lib:GetConditionalByCase(case)
	local r,e
	if case=='stance' then
		r=GetShapeshiftForm(true)
		e='UPDATE_SHAPESHIFT_FORM'
	elseif case=='talentgroup' then
		r=GetActiveTalentGroup(false,false)
		e='ACTIVE_TALENT_GROUP_CHANGED'
	--elseif case=='buff' then
	end
	return r,e
end

--adds a Frame to a Table an event runs-through on trigger and returns the "new" table
function lib:RegisterFrameForEvent(frameid,event,t)
	--create event in t if not already existent
	if not t[event] then
		t[event]={}
	end
	if not self:inTable(frameid,t[event]) then
		tinsert(t[event],frameid)
	end
	return t
end

function lib:UnregisterFrameForEvent(frameid,event,t)
	--create event in t if not already existent
	if not t[event] then return end
	local inTable,num = self:inTable(frameid,t[event])
	if inTable then tremove(t[event],num) end
	return t
end

function lib:UnregisterFrameForAllEvents(frameid,t)
	if #t>0 then
		for e in pairs(t) do
			self:UnregisterFrameForEvent(frameid,e,t)
		end
	end
end

function lib:GetCooldownByCmd(cmd,watch)
	if watch then
		if cmd=='spell' then
			return GetSpellCooldown(tonumber(watch) or watch)
		elseif cmd=='item' then
			return GetItemCooldown(tonumber(watch) or watch)
		elseif cmd=='flag' then
			return GetInventoryItemCooldown("player", watch)
		else 
			error(tostring(cmd)..' is an invalid Command')
		end
	else
		return 0,0,0
	end
end

function lib:GetRangeByCmd(cmd,input)
	if input then
		if cmd=='spell' then
			return IsSpellInRange(GetSpellInfo(input),"target")
		elseif cmd=='item' then
			return IsItemInRange(GetItemInfo(input),"target")
		elseif cmd=='flag' then
			local itemLink = GetInventoryItemLink("player", input)
			return IsItemInRange(itemLink, "target")
		else
			error(tostring(cmd)..' is an invalid Command')
		end
	end
end

--Function makes a copy of the table without the key
--and return this new tbl
function lib:DeleteFromTableByKey(tbl,value)
	local store={}
	for key,content in pairs(tbl) do
		if key~=value then
			store[key]=content
		end
	end
	return store
end


function lib:deepcopy(object)
    local lookup_table = {}
    local function _copy(object)
        if type(object) ~= "table" then
            return object
        elseif lookup_table[object] then
            return lookup_table[object]
        end
        local new_table = {}
        lookup_table[object] = new_table
        for index, value in pairs(object) do
            new_table[_copy(index)] = _copy(value)
        end
        return setmetatable(new_table, _copy(object))
    end
    return _copy(object)
end


local fade_in = {}
local fade_out= {}

--improved UIFrameFadeIn(frame,time,startAlpha,endAlpha,type)
function lib:UIFrameFadeIn(frame,t,startAlpha,endAlpha,typ)
	local now=GetTime()
	if typ=='in' then
		local IsInTable,num = self:inTable(frame.realname,fade_in)
		if not IsInTable then
			UIFrameFadeIn(frame,t,startAlpha,endAlpha)
			--tinsert(frame,{icon_name,endTime})
			tinsert(fade_in,{frame.realname,GetTime()+t})
		else
			if now>fade_in[num][2] then
				tremove(fade_in,num)
			end
		end
	elseif typ=='out' then
		local IsInTable,num = self:inTable(frame.realname,fade_out)
		if not IsInTable then
			UIFrameFadeIn(frame,t,startAlpha,endAlpha)
			tinsert(fade_out,{frame.realname,GetTime()+t})
		else
			if now>fade_in[num][2] then
				tremove(fade_out,num)
			end
		end
	end
end


